/*
 * PrettyFaces is an OpenSource JSF library to create bookmarkable URLs.
 * Copyright (C) 2009 - Lincoln Baxter, III <lincoln@ocpsoft.com> This program
 * is free software: you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version. This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see the file COPYING.LESSER
 * or visit the GNU website at <http://www.gnu.org/licenses/>.
 */
package com.ocpsoft.pretty.config.mapping;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author lb3
 */
public class QueryParameter extends RequestParameter
{
    private String name = "";
    private boolean decode = true;
    private String validatorIds = "";
    private String onError = "";

    public QueryParameter()
    {}

    public QueryParameter(final String name, final String expression)
    {
        this.name = name;
        setExpression(expression);
    }

    public boolean hasValidators()
    {
        return !validatorIds.trim().isEmpty();
    }

    public List<String> getValidatorIdList()
    {
        List<String> result = new ArrayList<String>();
        if (hasValidators())
        {
            String[] ids = validatorIds.split(" ");
            Collections.addAll(result, ids);
        }
        return result;
    }

    public String getName()
    {
        return name;
    }

    public void setName(final String name)
    {
        this.name = name;
    }

    public boolean decode()
    {
        return decode;
    }

    public void setDecode(final boolean decode)
    {
        this.decode = decode;
    }

    public String getValidatorIds()
    {
        return validatorIds;
    }

    public void setValidatorIds(final String validatorIds)
    {
        this.validatorIds = validatorIds;
    }

    public String getOnError()
    {
        return onError;
    }

    public void setOnError(final String onError)
    {
        this.onError = onError;
    }

    public boolean isDecode()
    {
        return decode;
    }

    @Override
    public int hashCode()
    {
        final int prime = 31;
        int result = super.hashCode();
        result = prime * result + (decode ? 1231 : 1237);
        result = prime * result + ((name == null) ? 0 : name.hashCode());
        result = prime * result + ((onError == null) ? 0 : onError.hashCode());
        result = prime * result + ((validatorIds == null) ? 0 : validatorIds.hashCode());
        return result;
    }

    @Override
    public boolean equals(final Object obj)
    {
        if (this == obj)
        {
            return true;
        }
        if (!super.equals(obj))
        {
            return false;
        }
        if (getClass() != obj.getClass())
        {
            return false;
        }
        QueryParameter other = (QueryParameter) obj;
        if (decode != other.decode)
        {
            return false;
        }
        if (name == null)
        {
            if (other.name != null)
            {
                return false;
            }
        }
        else if (!name.equals(other.name))
        {
            return false;
        }
        if (onError == null)
        {
            if (other.onError != null)
            {
                return false;
            }
        }
        else if (!onError.equals(other.onError))
        {
            return false;
        }
        if (validatorIds == null)
        {
            if (other.validatorIds != null)
            {
                return false;
            }
        }
        else if (!validatorIds.equals(other.validatorIds))
        {
            return false;
        }
        return true;
    }
}
